﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.IO.Ports;

namespace DesktopStation
{
    /// <summary>
    /// S88用イベントデータ
    /// </summary>
    /// <remarks>
    /// S88用イベントデータのクラス
    /// </remarks>
    public class S88EventCfgItem
    {
        public string mAreaName;
        public int mSensorAddress;
        //public int mLocAddress;
        public List<ScriptData> Items;
        public bool mAvailable;
        public int TriggerType;
        public int mIntervalTime;
        public int mTriggerTime;
        public int mLocIndex;
        public int mSlotIndex;

        public S88EventCfgItem()
        {
            mAreaName = "";
            mSensorAddress = 0;
            //mLocAddress = 0;
            mAvailable = false;
            TriggerType = 0;
            mIntervalTime = 0;
            mTriggerTime = 0;
            mLocIndex = -1;
            mSlotIndex = 0;

            Items = new List<ScriptData>();
        }
    } /* S88EventCfgItem */

    /// <summary>
    /// S88用イベントデータクラス
    /// </summary>
    /// <remarks>
    /// S88用イベントデータの保持に関する機能を持ったクラス
    /// </remarks>
    public class S88EventCfg
    {
        public S88EventCfgItem[] Events;

        public S88EventCfg()
        {
            int i;

            Events = new S88EventCfgItem[Program.S88EVENT_MAX];

            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                Events[i] = new S88EventCfgItem();
            }
        }
    } /* S88EventCfg */


    /// <summary>
    /// S88用イベントクラス
    /// </summary>
    /// <remarks>
    /// S88用イベントデータの実行に関する機能を持ったクラス
    /// </remarks>
    public class S88Event
    {
        public delegate int getScriptValue_delegate(int inNo);
        public delegate int getS88Value_delegate(int inNo);
        public delegate void setScriptValues_delegate(int inNo, int inValue);
        public delegate void writeS88ProcessLog_delegate(String inEventName, String inTrigger, int inLineNo, String inLog);
        
        public string EventName;
        public int mSensorAddress;
        public List<ScriptData> Items;
        public bool mDCCMode;
        public bool mAvailable;
        public int ScriptMode;
        public bool Run;
        public int TriggerType;
        public int mIntervalTime;
        public int mTriggerTime;
        public int mRandomTime;
        public DateTime mPreviousTime;
        public bool mRan;
        public int mLocIndex;
        public int mSlotIndex;

        private int lineNo;
        private int waitTime;

        private RailuinoSerial serialCmd;
        private getScriptValue_delegate getScriptValue;
        private setScriptValues_delegate setScriptValues;
        private UpdateLocList_delegate updateLocList;
        private UpdateAccList_delegate updateAccList;
        private writeS88ProcessLog_delegate writeS88ProcessLog;
        private runFile_delegate runFile;
        private setTransitionSpeed_delegate setTransitionSpeed;
        private getLocItemFromIndex_delegate getLocItemFromIndex;
        private getS88Value_delegate getS88Value;


        public S88Event(RailuinoSerial inSerialCmd, getScriptValue_delegate inGetScriptValue, setScriptValues_delegate inSetScriptValue, UpdateLocList_delegate inUpdateLocList, UpdateAccList_delegate inUpdateAccList, writeS88ProcessLog_delegate inWriteS88ProcessLog, runFile_delegate inRunFile, setTransitionSpeed_delegate inSetTransitionSpeed, getLocItemFromIndex_delegate inGetLocItemFromIndex, getS88Value_delegate inGetS88Value)
        {
            EventName = "";
            mSensorAddress = 0;
            mLocIndex = -1;
            mSlotIndex = 0;
            mAvailable = false;
            mDCCMode = false;
            Items = new List<ScriptData>();

            serialCmd = inSerialCmd;
            getScriptValue = inGetScriptValue;
            setScriptValues = inSetScriptValue;
            updateLocList = inUpdateLocList;
            updateAccList = inUpdateAccList;
            writeS88ProcessLog = inWriteS88ProcessLog;
            runFile = inRunFile;
            setTransitionSpeed = inSetTransitionSpeed;
            getLocItemFromIndex = inGetLocItemFromIndex;
            getS88Value = inGetS88Value;
               

            ScriptMode = Program.SCRIPTMODE_STOP;
            Run = false;
            mRan = false;

        }

        public void StartScript()
        {
            /* スクリプト処理開始 */
            ScriptMode = Program.SCRIPTMODE_RUN;
            waitTime = 0;
            lineNo = 0;
        }

        public void ExitScript()
        {
            /* スクリプト処理停止 */
            ScriptMode = Program.SCRIPTMODE_STOP;
        }

        /* 定周期でループ動作する処理 */
        public void RunScriptMain()
        {
            bool aBreak;

            if (ScriptMode != Program.SCRIPTMODE_RUN)
            {
                return;
            }

            if (waitTime <= 0)
            {
                while (lineNo < Items.Count)
                {

                    aBreak = ProcessScript(Items[lineNo]);

                    /* 次の行へ */
                    lineNo = lineNo + 1;

                    if (aBreak == true)
                    {
                        break;
                    }
                }

                if( ScriptMode == Program.SCRIPTMODE_RUN)
                {
                }

                if (Items.Count <= lineNo)
                {
                    if (Items[Items.Count - 1].mCommand != Program.SCRIPTCMD_EXIT)
                    {
                        ExitScript();

                        /* ログ */
                        writeS88ProcessLog(EventName, getTriggerText(), 9999, "Force Exit");
                    }

                    ScriptMode = Program.SCRIPTMODE_STOP;
                    lineNo = 0;
                }

            }
            else
            {

                /* 待機する */
                waitTime = waitTime - 1;

            }
        }

        private String getTriggerText()
        {
            String aText = "";

            switch (TriggerType)
            {
                case 0:
                case 1:
                case 2:
                case 3:
                    aText = "SAd." + mSensorAddress.ToString();
                    break;
                case 4:
                    aText = "Int." + (mIntervalTime / (60 * 60)).ToString("D2") + ":" + ((mIntervalTime / 60) % 60).ToString("D2") + ":" + (mIntervalTime % 60).ToString("D2");
                    break;
                case 5:
                    aText = "Clk." + (mTriggerTime / (60 * 60)).ToString("D2") + ":" + ((mTriggerTime / 60) % 60).ToString("D2") + ":" + (mTriggerTime % 60).ToString("D2");
                    break;
            }

            return aText;
        }

        private bool ProcessScript(ScriptData inData)
        {
            bool aResult = false;
            int aTemp;
            int aLocAddress;
            int aLineNo;
            int aProtocol = 0;
            int aSpeedRatio = 1024;
            LocData aLocData;

            if (inData.mCommand == Program.SCRIPTCMD_WAIT)
            {
                waitTime = convertParam(inData.mParam1);
                aResult = true;

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_SPEED)
            {
                aLocAddress = convertParam(inData.mParam1);

                /* 選択された機関車を使用する */
                if (aLocAddress <= 0)
                {
                    aLocData = getLocItemFromIndex(mLocIndex, mSlotIndex);
                    if (aLocData != null)
                    {
                        aLocAddress = aLocData.mLocAddr;
                        aProtocol = aLocData.mLocSpeedstep;
                        aSpeedRatio = aLocData.mLocMaxSpeed + 1;
                    }
                }
                else
                {
                    aLocData = null;
                }

                if (inData.mParam3 == 0)
                {
                    serialCmd.SetLocoSpeed(aLocAddress, (convertParam(inData.mParam2) * aSpeedRatio) >> 10, aProtocol);

                    /* 管理リストに反映 */
                    updateLocList(aLocAddress, (convertParam(inData.mParam2) * aSpeedRatio) >> 10, -1, -1, -1);

                    /* 重連運転想定 */
                    if (aLocData != null)
                    {
                        if (aLocData.mLocAddr_dbl > 0)
                        {
                            serialCmd.SetLocoSpeed(aLocData.mLocAddr_dbl, (convertParam(inData.mParam2) * aSpeedRatio) >> 10, aLocData.mDoubleLoc[0].mLocSpeedstep);
                            /* 管理リストに反映 */
                            updateLocList(aLocData.mLocAddr_dbl, (convertParam(inData.mParam2) * aSpeedRatio) >> 10, -1, -1, -1);

                        }
                    }
                }
                else
                {
                    waitTime = inData.mParam3;
                    aResult = true;

                    setTransitionSpeed(aLocAddress, (convertParam(inData.mParam2) * aSpeedRatio) >> 10, waitTime);

                    /* 管理リストには自動で反映 */
                }

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + aLocAddress.ToString() + "," + inData.mParam2 + "," + inData.mParam3.ToString() + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_DIRECTION)
            {
                aLocAddress = convertParam(inData.mParam1);

                if (aLocAddress <= 0)
                {
                    aLocData = getLocItemFromIndex(mLocIndex, mSlotIndex);

                    if (aLocData != null)
                    {
                        aLocAddress = aLocData.mLocAddr;
                    }
                }
                else
                {
                    aLocData = null;
                }

                serialCmd.SetLocoDirection(aLocAddress, convertParam(inData.mParam2));
                /* 管理リストに反映 */
                updateLocList(aLocAddress, -1, convertParam(inData.mParam2), -1, -1);


                /* 重連運転想定 */
                if (aLocData != null)
                {
                    if (aLocData.mLocAddr_dbl > 0)
                    {
                        serialCmd.SetLocoDirection(aLocData.mLocAddr_dbl, convertParam(inData.mParam2));
                        /* 管理リストに反映 */
                        updateLocList(aLocData.mLocAddr_dbl, -1, convertParam(inData.mParam2), -1, -1);
                    }
                }

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + aLocAddress.ToString() + "," + inData.mParam2 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_FUNCTION)
            {
                aLocAddress = convertParam(inData.mParam1);

                if (aLocAddress <= 0)
                {
                    aLocData = getLocItemFromIndex(mLocIndex, mSlotIndex);

                    if (aLocData != null)
                    {
                        aLocAddress = aLocData.mLocAddr;
                    }
                }

                serialCmd.SetLocoFunction(aLocAddress, convertParam(inData.mParam2), inData.mParam3);

                /* 管理リストに反映 */
                updateLocList(aLocAddress, -1, -1, convertParam(inData.mParam2), inData.mParam3);

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + aLocAddress.ToString() + "," + inData.mParam2 + "," + inData.mParam3 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_ACCESSORY)
            {
                serialCmd.SetTurnout(convertParam(inData.mParam1), convertParam(inData.mParam2), mDCCMode);

                /* 管理リストに反映 */
                updateAccList(convertParam(inData.mParam1), convertParam(inData.mParam2));

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_POWER)
            {
                serialCmd.SetPower(convertParam(inData.mParam1));

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_EXIT)
            {
                ExitScript();
                /* 処理中断 */
                aResult = true;

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "()");
            }

            if (inData.mCommand == Program.SCRIPTCMD_GOTO)
            {
                /* 行番号を退避 */
                aLineNo = lineNo;

                lineNo = convertParam(inData.mParam1) - 1;

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_GOTOLABEL)
            {
                /* 行番号を退避 */
                aLineNo = lineNo;

                /* 指定ラベルへジャンプする */
                aTemp = searchLabel(inData.mParam1);
                if (aTemp >= 0)
                {
                    lineNo = aTemp - 1;
                }

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + " -> " + lineNo.ToString() + ")");
            }

            if (inData.mCommand == Program.SCRIPTCMD_SETFLAG)
            {
                setScriptValues(convertParam(inData.mParam1), convertParam(inData.mParam2));
                
                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + ")");
            }

            /* フラグの値を加算（減算）する */
            if (inData.mCommand == Program.SCRIPTCMD_INCFLAG)
            {
                aTemp = getScriptValue(convertParam(inData.mParam1));

                if (convertParam(inData.mParam2) == 0)
                {
                    aTemp = aTemp + 1;
                }
                else
                {
                    aTemp = aTemp + convertParam(inData.mParam2);
                }

                setScriptValues(convertParam(inData.mParam1), aTemp);
                
                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + " -> " + aTemp.ToString() + ")");
            }


            if (inData.mCommand == Program.SCRIPTCMD_JUMP)
            {
                /* 行番号を退避 */
                aLineNo = lineNo;

                /* FLAGの状態に応じて指定ラベルへジャンプする */
                if (getScriptValue(convertParam(inData.mParam2)) == inData.mParam3)
                {
                    aTemp = searchLabel(inData.mParam1);

                    if (aTemp >= 0)
                    {
                        lineNo = aTemp;
                    }

                    /* ログ */
                    writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + "," + inData.mParam3 + " -> " + aTemp.ToString() + ")");
                }
                else
                {
                    /* ログ */
                    writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + "," + inData.mParam3 + ")");
                }

            }

            if (inData.mCommand == Program.SCRIPTCMD_JUMPS88)
            {
                /* 行番号を退避 */
                aLineNo = lineNo;

                int aS88status_jump = getS88Value(convertParam(inData.mParam2));

                /* FLAGの状態に応じて指定ラベルへジャンプする */
                if (aS88status_jump == inData.mParam3)
                {
                    aTemp = searchLabel(inData.mParam1);

                    if (aTemp >= 0)
                    {
                        lineNo = aTemp;
                    }

                    /* ログ */
                    writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + "," + inData.mParam3 + " -> " + aTemp.ToString() + ")");
                }
                else
                {
                    /* ログ */
                    writeS88ProcessLog(EventName, getTriggerText(), aLineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + "," + inData.mParam3 + ")");
                }

            }


            if (inData.mCommand == Program.SCRIPTCMD_LABEL)
            {
                /* ラベルなので何もしない */
            }

            if (inData.mCommand == Program.SCRIPTCMD_RUNFILE)
            {
                /* ファイル実行 */
                runFile(inData.mParam1, inData.mParam2);

                /* ログ */
                writeS88ProcessLog(EventName, getTriggerText(), lineNo, inData.mCommand + "(" + inData.mParam1 + "," + inData.mParam2 + ")");
            } 


            return aResult;
        }

        private int convertParam(String inText)
        {
            int aResult = 0;

            if (inText == "")
            {
                return 0;
            }

            Int32.TryParse(inText, out aResult);

            return aResult;
        }

        private int searchLabel(String inLabelName)
        {
            int aResult = -1;
            int i;

            for (i = 0; i < Items.Count; i++)
            {
                if (Items[i].mCommand == Program.SCRIPTCMD_LABEL)
                {
                    if (Items[i].mParam1 == inLabelName)
                    {
                        aResult = i;
                        break;
                    }
                }
            }


            return aResult;
        }



    } /* S88Event */


    /// <summary>
    /// S88用イベント管理クラス
    /// </summary>
    /// <remarks>
    /// S88用イベント処理に関する管理機能を持ったクラス
    /// </remarks>
    /// 
    public class S88EventManager
    {
        public S88Event[] mEvents;

        public int Mode;
        private int[] lastFlags;
        private RailuinoSerial serialCmd;
        private int[] valuesScript;

        private SetS88RunText_delegate setS88RunText;
        Random RandomFunc;

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="inSerialCmd">Railuinoシリアルコマンド出力クラス</param>
        /// <param name="inUpdateLocList">機関車データ一覧更新関数</param>
        /// <param name="inUpdateAccList">アクセサリデータ一覧更新関数</param>
        public S88EventManager(RailuinoSerial inSerialCmd, UpdateLocList_delegate inUpdateLocList, UpdateAccList_delegate inUpdateAccList, SetS88RunText_delegate inSetS88RunText, runFile_delegate inRunFile, setTransitionSpeed_delegate inSetTransitionSpeed, getLocItemFromIndex_delegate inGetLocItemFromIndex)
        {
            int i;

            mEvents = new S88Event[Program.S88EVENT_MAX];
            lastFlags = new int[Program.S88_MAX];

            valuesScript = new int[Program.SCRIPTVALUE_MAX];
            /* Init values on script.*/
            for (i = 0; i < Program.SCRIPTVALUE_MAX; i++)
            {
                valuesScript[i] = 0;
            }
            
            serialCmd = inSerialCmd;
            setS88RunText = inSetS88RunText;

            /* Generate class */
            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                mEvents[i] = new S88Event(inSerialCmd, GetScriptValues, setScriptValues, inUpdateLocList, inUpdateAccList, writeS88ProcessLog, inRunFile, inSetTransitionSpeed, inGetLocItemFromIndex, GetS88Values);
            }

            RandomFunc = new System.Random();


        }

         /// <summary>
         /// 初期化
         /// </summary>
         /// <param name="inDCCmode">DCCモード使用フラグ</param>

        public void Clear(bool inDCCmode)
        {
            int i;

            /* Init values on script.*/
            for (i = 0; i < Program.SCRIPTVALUE_MAX; i++)
            {
                valuesScript[i] = 0;
            }

            /* S88センサのバッファも初期化 */
            for (i = 0; i < Program.S88_MAX; i++)
            {
                /* 最初からセンサ上に機関車がいる場合を想定して初期値は0にしない。 */
                lastFlags[i] = -1;
            }

            /* DCCモードを使用するかどうかの設定をイベントデータにセット */
            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                mEvents[i].mDCCMode = inDCCmode;
            }

        }

        /// <summary>
        /// 前回センサ状態取得
        /// </summary>
        /// <param name="inNo">インデックス番号(0-16)</param>
        /// 
        public int GetSensorLastFlag(int inNo)
        {
            return lastFlags[inNo];
        }

        /// <summary>
        /// フラグ値取得
        /// </summary>
        /// <param name="inNo">フラグ番号(0-9)</param>
        /// 
        public int GetScriptValues(int inNo)
        {
            if ((inNo < Program.SCRIPTVALUE_MAX) && (inNo >= 0))
            {
                return valuesScript[inNo];
            }
            else
            {
                return 0;
            }
        }

        private void setScriptValues(int inNo, int inValue)
        {
            if ((inNo < Program.SCRIPTVALUE_MAX) && (inNo >= 0))
            {
                valuesScript[inNo] = inValue;
            } 

        }

        /// <summary>
        /// S88値取得
        /// </summary>
        /// <param name="inNo">S88番号(0-255)</param>
        /// 
        public int GetS88Values(int inNo)
        {
            int aFlagHigh;
            int aFlagLow;
            
            if ((inNo < Program.S88_MAX) && (inNo >= 0))
            {
                aFlagHigh = inNo / 16;
                aFlagLow = inNo % 16;

                if (lastFlags[aFlagHigh] == -1)
                {
                    return 0;
                }
                else
                {
                    return (lastFlags[aFlagHigh] >> aFlagLow) & 0x01;
                }
            }
            else
            {
                return 0;
            }
        }


        /// <summary>
        /// センサ更新チェック
        /// </summary>
        /// <param name="inFlags">S88センサ状態データ配列</param>
        ///        
        public bool UpdateCheck(int[] inFlags)
        {
            int i;
            int aUpdateNums = 0;
 

            for (i = 0; i < Program.S88_MAX; i++)
            {
                /* 前回値を保存する */
                if ((lastFlags[i] != inFlags[i]) && (lastFlags[i] >= 0))
                {
                    aUpdateNums = aUpdateNums + 1;
                }
            }

            return (aUpdateNums > 0 ? true : false);
        }
        /// <summary>
        /// センサチェックとスクリプト実行タイミングチェック(入線・退線の変化を検出)
        /// </summary>
        /// <param name="inFlags">S88センサ状態データ配列</param>
        ///        
        public bool IntervalCheck(int[] inFlags)
        {
            int i;
            int aCheckFlag;
            int aUpdateNums = 0;
            int aFlagHigh;
            int aFlagLow;
            int aFlagLast;
            int aFlagCurrent;

            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                /* センサアドレスが有効で、イベント動作フラグONのとき */
                if ((mEvents[i].mSensorAddress > 0) && (mEvents[i].mAvailable == true) && (mEvents[i].Run == false))
                {
                    int aSensorAddress = mEvents[i].mSensorAddress - 1;

                    /* センサアドレスから、センサスレーブNoとビット位置に換算する */
                    aFlagHigh = aSensorAddress / 16;
                    aFlagLow = aSensorAddress % 16;

                    /* 前回値が-1(初期値)ではないとき */
                    if (lastFlags[aFlagHigh] >= 0)
                    {
                        /* 新旧のフラグ値のみを取得 */
                        aFlagLast = (lastFlags[aFlagHigh] >> aFlagLow) & 0x01;
                        aFlagCurrent = (inFlags[aFlagHigh] >> aFlagLow) & 0x01;

                        aCheckFlag = aFlagCurrent - aFlagLast;

                        // 進入のとき
                        if ((aCheckFlag > 0) && (mEvents[i].TriggerType == 0))
                        {
                            mEvents[i].Run = true;

                        }
                        else if ((aCheckFlag < 0) && (mEvents[i].TriggerType == 1))
                        {
                            // Negative Edgeのとき
                            mEvents[i].Run = true;

                        }

                        /* 変化チェック用（表示関連） */
                        if (aCheckFlag != 0)
                        {
                            aUpdateNums = aUpdateNums + 1;
                        }
                    }
                }
            }

            for (i = 0; i < Program.S88_MAX; i++)
            {
                /* 前回値を保存する */
                lastFlags[i] = inFlags[i];
            }

            /* 返す値は変化のあったイベントの数 */
            return (aUpdateNums > 0 ? true : false);
        }

        /// <summary>
        /// センサチェックとスクリプト実行タイミングチェック（1sec周期,その他の条件を監視）
        /// </summary>
        /// <param name="inFlags">S88センサ状態データ配列</param>
        /// <param name="inCurrentTime">現在時刻</param>
        ///        
        public bool IntervalCheckTime(int[] inFlags, DateTime inCurrentTime, LocomotiveDB inLocDB)
        {
            int i;
            int aUpdateNums = 0;
            int aFlagHigh;
            int aFlagLow;
            int aFlagCurrent;
            int aLocIndex; 
            TimeSpan aDifference;


            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                /* センサアドレスが有効で、イベント動作フラグONのとき */
                if ((mEvents[i].TriggerType >= 2) && (mEvents[i].mAvailable == true) && (mEvents[i].Run == false))
                {

                    switch (mEvents[i].TriggerType)
                    {

                        case 2:
                        case 3:
                            /* 在線チェック */

                            /* センサアドレスから、センサスレーブNoとビット位置に換算する */
                            aFlagHigh = mEvents[i].mSensorAddress / 16;
                            aFlagLow = mEvents[i].mSensorAddress - aFlagHigh * 16 - 1;
                            aFlagCurrent = (inFlags[aFlagHigh] >> aFlagLow) & 0x01;

                            if( (mEvents[i].TriggerType == 2) && (aFlagCurrent == 1) )
                            {
                                /* 在線しているとき */
                                if (mEvents[i].mRan == false)
                                {
                                    mEvents[i].Run = true;
                                    mEvents[i].mRan = true;
                                }

                                /* 変化チェック用（表示関連） */
                                aUpdateNums = aUpdateNums + 1;
                            }
                            else if ((mEvents[i].TriggerType == 3) && (aFlagCurrent == 0) )
                            {
                                /* 在線していないとき */
                                if (mEvents[i].mRan == false)
                                {
                                    mEvents[i].Run = true;
                                    mEvents[i].mRan = true;
                                }

                                /* 変化チェック用（表示関連） */
                                aUpdateNums = aUpdateNums + 1;
                            }
                            else
                            {
                                mEvents[i].mRan = false;
                            }

                            break;

                        case 4:
                            /* Intervalチェック */

                            if (mEvents[i].mPreviousTime.CompareTo(DateTime.MinValue) == 0)
                            {
                                mEvents[i].mPreviousTime = inCurrentTime;
                            }

                            aDifference = inCurrentTime - mEvents[i].mPreviousTime;

                            if (aDifference.TotalSeconds >= mEvents[i].mIntervalTime)
                            {
                                mEvents[i].Run = true;
                                mEvents[i].mPreviousTime = inCurrentTime;

                                /* 変化チェック用（表示関連） */
                                aUpdateNums = aUpdateNums + 1;

                            }
                            else
                            {
                                mEvents[i].mRan = false;
                            }


                            break;

                        case 5:
                            /* Clockチェック */

                            int aTemp = (inCurrentTime.Hour * 60 * 60) + (inCurrentTime.Minute * 60) + (inCurrentTime.Second);

                            if (Math.Abs(mEvents[i].mTriggerTime - aTemp) <= 0)
                            {
                                if (mEvents[i].mRan == false)
                                {
                                    mEvents[i].Run = true;
                                    mEvents[i].mRan = true;
                                }
                            }
                            else
                            {
                                mEvents[i].mRan = false;
                            }

                            /* 変化チェック用（表示関連） */
                            aUpdateNums = aUpdateNums + 1;

                            break;
                        case 6:
                            /* Random Intervalチェック */

                            if (mEvents[i].mPreviousTime.CompareTo(DateTime.MinValue) == 0)
                            {
                                mEvents[i].mPreviousTime = inCurrentTime;
                                mEvents[i].mRandomTime = RandomFunc.Next(mEvents[i].mIntervalTime);
                                
                            }

                            aDifference = inCurrentTime - mEvents[i].mPreviousTime;

                            if (aDifference.TotalSeconds >= mEvents[i].mRandomTime)
                            {
                                mEvents[i].Run = true;
                                mEvents[i].mPreviousTime = inCurrentTime;
                                mEvents[i].mRandomTime = RandomFunc.Next(mEvents[i].mIntervalTime);

                                /* 変化チェック用（表示関連） */
                                aUpdateNums = aUpdateNums + 1;


                            }
                            else
                            {
                                mEvents[i].mRan = false;
                            }

                            break;

                        case 7:
                            /* 車両運転状況チェック（走行時） */

                            aLocIndex = mEvents[i].mLocIndex;

                            if (aLocIndex >= 0)
                            {

                                if (inLocDB.Items[aLocIndex].mCurrentSpeed > 102)
                                {
                                    mEvents[i].Run = true;
                                    /* 変化チェック用（表示関連） */
                                    aUpdateNums = aUpdateNums + 1;

                                }
                                else
                                {
                                    mEvents[i].mRan = false;
                                }

                            }

                            break;

                        case 8:
                            /* 車両運転状況チェック（停止時） */

                            aLocIndex = mEvents[i].mLocIndex;

                            if (aLocIndex >= 0)
                            {

                                if (inLocDB.Items[aLocIndex].mCurrentSpeed <= 0 )
                                {
                                    mEvents[i].Run = true;
                                    /* 変化チェック用（表示関連） */
                                    aUpdateNums = aUpdateNums + 1;
                                }
                                else
                                {
                                    mEvents[i].mRan = false;
                                }


                            }

                            break;                    

                    }

                }

            }

            /* 返す値は変化のあったイベントの数 */
            return (aUpdateNums > 0 ? true : false);
        }

        /// <summary>
        /// 100ms周期スクリプト実行
        /// </summary>
        public void IntervalRun()
        {
            int i;

            /* 実行処理 */
            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                /* センサアドレスが有効で、イベント動作フラグONのとき */
                if (mEvents[i].Run == true)
                {
                    /* START check */
                    if (mEvents[i].ScriptMode == Program.SCRIPTMODE_STOP)
                    {
                        mEvents[i].StartScript();
                    }

                    mEvents[i].RunScriptMain();

                    /* STOP check */
                    if (mEvents[i].ScriptMode == Program.SCRIPTMODE_STOP)
                    {
                        mEvents[i].Run = false;
                    }
                 }
            }
        }

        public void SaveToFile(String inFilename)
        {
            int i, j;

            //いったん保存領域に展開する
            S88EventCfg aTempCfg = new S88EventCfg();

            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                aTempCfg.Events[i].mSensorAddress = mEvents[i].mSensorAddress;
                aTempCfg.Events[i].mAreaName = mEvents[i].EventName;
                aTempCfg.Events[i].mAvailable = mEvents[i].mAvailable;
                aTempCfg.Events[i].mLocIndex = mEvents[i].mLocIndex;
                aTempCfg.Events[i].mSlotIndex = mEvents[i].mSlotIndex;

                aTempCfg.Events[i].TriggerType = mEvents[i].TriggerType;
                aTempCfg.Events[i].mIntervalTime = mEvents[i].mIntervalTime;
                aTempCfg.Events[i].mTriggerTime = mEvents[i].mTriggerTime;
               

                for (j = 0; j < mEvents[i].Items.Count; j++)
                {
                    ScriptData aItem = new ScriptData();
                    aItem.mCommand = mEvents[i].Items[j].mCommand;
                    aItem.mParam1 = mEvents[i].Items[j].mParam1;
                    aItem.mParam2 = mEvents[i].Items[j].mParam2;
                    aItem.mParam3 = mEvents[i].Items[j].mParam3;
                    aItem.mParam4 = mEvents[i].Items[j].mParam4;
                    aTempCfg.Events[i].Items.Add(aItem);

                }
            }

            //XmlSerializerを呼び出す
            XmlSerializer serializer = new XmlSerializer(typeof(S88EventCfg));
            //ファイルを作る
            FileStream fs = new FileStream(inFilename, FileMode.Create);
            //書き込み
            serializer.Serialize(fs, aTempCfg);  //sclsはSampleClassのインスタンス名
            //ファイルを閉じる
            fs.Close();
        }

        public bool LoadFromFile(String inFilename)
        {
            bool retVal = false;
            int i, j;

            /* 初期化 */
            for (i = 0; i < Program.S88EVENT_MAX; i++)
            {
                mEvents[i].Items.Clear();
                mEvents[i].EventName = "";
                mEvents[i].mSensorAddress = 0;
                mEvents[i].mLocIndex = -1;
                mEvents[i].mSlotIndex = 0;
                mEvents[i].mAvailable = false;

            }


            //設定ファイルがあるときのみ読み込みします。
            if (File.Exists(inFilename))
            {
                //XmlSerializerを呼び出す
                XmlSerializer serializer = new XmlSerializer(typeof(S88EventCfg));
                //ファイルを開く
                FileStream fs = new FileStream(inFilename, FileMode.Open);

                //読み込み
                S88EventCfg aTempCfg = new S88EventCfg();
                aTempCfg = (S88EventCfg)serializer.Deserialize(fs);
                //ファイルを閉じる
                fs.Close();

                //反映
                for (i = 0; i < Program.S88EVENT_MAX; i++)
                {
                    mEvents[i].mSensorAddress = aTempCfg.Events[i].mSensorAddress;
                    mEvents[i].EventName = aTempCfg.Events[i].mAreaName;
                    mEvents[i].mAvailable = aTempCfg.Events[i].mAvailable;
                    mEvents[i].mLocIndex = aTempCfg.Events[i].mLocIndex;
                    mEvents[i].mSlotIndex = aTempCfg.Events[i].mSlotIndex;
                    mEvents[i].TriggerType = aTempCfg.Events[i].TriggerType;
                    mEvents[i].mIntervalTime = aTempCfg.Events[i].mIntervalTime;
                    mEvents[i].mTriggerTime = aTempCfg.Events[i].mTriggerTime;

                    for (j = 0; j < aTempCfg.Events[i].Items.Count; j++)
                    {
                        ScriptData aItem = new ScriptData();
                        aItem.mCommand = aTempCfg.Events[i].Items[j].mCommand;
                        aItem.mParam1 = aTempCfg.Events[i].Items[j].mParam1;
                        aItem.mParam2 = aTempCfg.Events[i].Items[j].mParam2;
                        aItem.mParam3 = aTempCfg.Events[i].Items[j].mParam3;
                        aItem.mParam4 = aTempCfg.Events[i].Items[j].mParam4;
                        mEvents[i].Items.Add(aItem);

                    }
                }


                retVal = true;
            }
            else
            {
                /* 初期化は完了済 */
            }


            return retVal;
        }

        private void writeS88ProcessLog(String inEventName, String inTrigger, int inLineNo, String inLog)
        {

            setS88RunText("[" + inEventName + "(" + inTrigger + "), " + inLineNo.ToString() + "] " + inLog);

        }


    } /* S88EventManager */

}
